package cz.cvut.ok.kapo.paa.kybl;

import java.util.LinkedList;
import java.util.List;


/**
 * Reprezentuje jeden uzel stavoveho prostoru.
 * @author karel
 *
 */
public class Node {
	private int visits;
	private Node parent;
	private int length; //nastavuje automaticky setParent()
	private int opcode;	// cislo operace, jak se z parenta dostal sem (class OpCodes)
	private int srcKybl, dstKybl;	//cislo kyblu, ze ktereho, a kam.
	private boolean isEndState;
	
	public Node() {
		super();
		this.visits=0;
		this.length=0;
		this.parent=null;
		this.dstKybl=-1;
		this.isEndState=false;
		this.opcode=OpCodes.OC_NONE;
		this.srcKybl=-1;
	}

	/**
	 * @return Returns the length.
	 */
	public int getLength() {
		return length;
	}

	/**
	 * @return Returns the parent.
	 */
	public Node getParent() {
		return parent;
	}

	/**
	 * @param parent The parent to set.
	 */
	public void setParent(Node parent) {
		this.parent = parent;
		if (parent!=null)
			this.length = parent.getLength()+1;
		else
			this.length=0;
	}
	
	/**
	 * Ulozi zpusob, jakym se z parenta stal tento uzel
	 * @param srcKybl - pokud je operace unarni, byla pouzita na src kybl.
	 * @param dstKybl
	 * @param opcode
	 */
	public void setOperation(int srcKybl, int dstKybl, int opcode){
		this.srcKybl=srcKybl;
		this.dstKybl=dstKybl;
		this.opcode=opcode;
	}

	
	/**
	 * @return Returns the opcode.
	 */
	public int getOpcode() {
		return opcode;
	}

	/**
	 * @return Returns the dstKybl.
	 */
	public int getDstKybl() {
		return dstKybl;
	}

	/**
	 * @return Returns the srcKybl.
	 */
	public int getSrcKybl() {
		return srcKybl;
	}

	/**
	 * @return Returns the visits.
	 */
	public int getVisits() {
		return visits;
	}

	/**
	 * @return Has node been visited?
	 */
	public boolean isVisited(){
		return this.visits>0;
	}
	
	/**
	 * Increments visits of node by 1.
	 */
	public void incrementVisits() {
		this.visits++;
	}
	
	/**
	 * Vrati komplet cestu. S tim, ze startovaci pozice
	 * je na zacatku seznamu.
	 * @return
	 */
	public List<Node> getPath(){
		LinkedList<Node> ll = new LinkedList<Node>();
		Node tmp=this;
		while (tmp!=null){
			ll.addFirst(tmp);
			tmp = tmp.getParent();
		}
		return ll;
	}

	/**
	 * @return Returns the isEndState.
	 */
	public boolean isEndState() {
		return isEndState;
	}

	/**
	 * @param isEndState The isEndState to set.
	 */
	public void setEndState(boolean isEndState) {
		this.isEndState = isEndState;
	}

}
