package cz.cvut.ok.kapo.paa.kybl;


/**
 * Slouzi k manipulaci s kybly pres jejich stavy. Tj. provadi povolene
 * operace: napln, vylij, prelij.
 * @author karel
 *
 */
public class StateManipulator {
	private Configuration configuration;
	
	public StateManipulator(Configuration conf) {
		super();
		this.configuration=conf;
	}
	
	/**
	 * Naplni kybl na max.
	 */
	public boolean fill(State state, int kybl){
		if (state.getKybl(kybl)==configuration.getCapacities().getKybl(kybl))
			return false;
		state.setKybl(kybl, configuration.getCapacities().getKybl(kybl));
		return true;
	}

	/**
	 * Vyprazdni kybl
	 */
	public boolean empty(State state, int kybl){
		if (state.getKybl(kybl)==0)
			return false;
		state.setKybl(kybl, 0);
		return true;
	}
	
	/**
	 * Prelije src kybl do dest kyblu.
	 * @return true, pokud to jde, false, pokud ne (napr. prazdny src kybl, plny dst kybl)
	 */
	public boolean pour(State state, int src, int dst){
		int tmp;
		tmp = configuration.getCapacities().getKybl(dst) - state.getKybl(dst); 
		if (tmp>state.getKybl(src))
			tmp=state.getKybl(src);
		if (tmp==0)
			return false;
		state.setKybl(dst,state.getKybl(dst)+tmp);
		state.setKybl(src,state.getKybl(src)-tmp);
		return true;
	}
	
	/**
	 * Aplikuje operazi z uzlu na dany stav.
	 * @param state stav, na ktery ma byt aplikovana operace
	 * @param operation uzel, ktery v sobe  obsahuje operaci jakou vznikl.
	 * @return
	 */
	public boolean applyOperation(State state, Node operation){
		switch(operation.getOpcode()){
			case OpCodes.OC_FILL:
				return fill(state, operation.getSrcKybl()); 
			case OpCodes.OC_EMPTY:
				return empty(state, operation.getSrcKybl());
			case OpCodes.OC_POUR:
				return pour(state, operation.getSrcKybl(), operation.getDstKybl());
			case OpCodes.OC_NONE:
			default:
				break;
		}
		return true;
	}

}
