package cz.cvut.ok.kapo.paa.kybl.engines.bfs;

import java.util.List;

import cz.cvut.ok.kapo.paa.kybl.Configuration;
import cz.cvut.ok.kapo.paa.kybl.Node;
import cz.cvut.ok.kapo.paa.kybl.OpCodes;
import cz.cvut.ok.kapo.paa.kybl.State;
import cz.cvut.ok.kapo.paa.kybl.StateManipulator;
import cz.cvut.ok.kapo.paa.kybl.StateSpace;
import cz.cvut.ok.kapo.paa.kybl.engines.Engine;
import cz.cvut.ok.kapo.paa.kybl.utils.MyLinkedList;
import cz.cvut.ok.kapo.paa.kybl.utils.MyQueue;

/**
 * Prohledavani do sirky.
 * @author karel
 *
 */
public class BFSEngine extends Engine {
	protected MyQueue<State> pq;
	protected StateManipulator sm;

	public BFSEngine(Configuration conf) {
		super(conf);
		this.sm=new StateManipulator(conf);
		this.pq = new MyLinkedList<State>();
		//nastavime prvni nod do stavoveho prostoru a hodime ho i na zasobnik
		this.stateSpace.setNode(configuration.getStartState(), new Node());
		this.pq.add(configuration.getStartState());
	}

	/**
	 * Otestuje predany stav, zda jiz neexistuje
	 * ve stavovem prostoru. Pokud existuje, vraci false,
	 * protoze ho tam nekdo pred nami dal. Nezmusi mit ani nastaveny 
	 * visited. Pokud neexistuje, vytvori odpovidajici novy uzel
	 * a ulozi ho.
	 * @param state
	 * @return
	 */
	protected boolean saveNode(State state, Node parent, int opcode, int src, int dst){
		Node tmp = stateSpace.getNode(state);
		if (tmp==null){	//pokud tam nic neexistuje, musime to vytvorit
			tmp = new Node();
			tmp.setParent(parent);
			tmp.setOperation(src, dst,opcode);
			stateSpace.setNode(state, tmp);	//ulozim novy node do site
			if (state.equals(configuration.getEndState()))
				pathExist=true;
			return true;
		}
		// uzel v siti jiz existuje
		return false;
	}
	
	/**
	 * vygeneruje seznam novych stavu a rovnou ho vlozi do fronty
	 * a zaroven i aplikuje na stavovy prostor.
	 */
	protected void generateNewStates(State actual, Node actualNode){
		//napred vygenerujeme vsechny mozne naplneni/vyprazdneni. 
		//budem to delat napreskacku. Muzem pak vyzkouset, jak se to bude chovat v pripade
		//ruznych kombinaci. samozrejme je to datove zavisle.
		State tmp;
		for (int i=1; i<=5; i++){	//cislovani kyblu mame od 1 do 5
			tmp = actual.clone();
			if (sm.empty(tmp,i))	//operace EMPTY
				if (saveNode(tmp,actualNode, OpCodes.OC_EMPTY,i,-1))
					pq.add(tmp);	//pokud uzel nebyl v siti, pridam ho do fronty
			
			tmp = actual.clone();
			if (sm.fill(tmp,i))	//operace FILL
				if (saveNode(tmp,actualNode, OpCodes.OC_FILL,i,-1))
					pq.add(tmp);	//pokud uzel nebyl v siti, pridam ho do fronty

			for (int j=1;j<=5;j++){
				tmp = actual.clone();
				if (sm.pour(tmp,i,j))	//operace preliti (POUR)
					if (saveNode(tmp,actualNode, OpCodes.OC_POUR,i,j))
						pq.add(tmp);	//pokud uzel nebyl v siti, pridam ho do fronty
			}
		}
	}
	
	/**
	 * Projde stavovy prostor do sirky a najde nejkratsi reseni
	 * problemu prelevani kyblu.
	 */
	@Override
	public void execute() {
		State actualState;
		Node actualNode;
		
		while((actualState = pq.poll())!=null && !existPath()){
			actualNode = stateSpace.getNode(actualState);
			if (actualNode.isVisited())	//preskocime jiz navstiveny uzel
				continue;
/*			if (actualNode)	{//vytvorili jsme cestu az ke koncovemu uzlu, muzeme skoncit
				pathExist=true;
				break;
			}*/
			actualNode.incrementVisits();	//inkrementujeme uzel: prosli jsme ho.
			// vygenerovat nasledovniky, nasazet je do fronty a vytvorit nody
			generateNewStates(actualState,actualNode);
		}
		//vycistime frontu a vypiseme statistiku (zatim)
		System.out.println("inserts("+pq.getInserts()+") removes("+pq.getRemoves()+")");
		pq.clear();
	}
	
	/**
	 * @return Vrati cestu od zacatku do konce jako seznam.
	 * Cesta obsahuje seznam Nodu, jez obsahuji operace pro presun do dalsiho stavu.
	 */
	@Override
	public List<Node> getPath(){
		if (pathExist)
			return getStateSpace().getEndNode().getPath();
		return null;
	}
	
	/**
	 * Pouze kvli testovani (aby bylo videt v testcase-u).
	 * @return Returns the stateSpace.
	 */
	@Override
	protected StateSpace getStateSpace(){
		return stateSpace;
	}

}
