package cz.cvut.ok.kapo.paa.kybl.engines.bfs;

import java.io.StringWriter;
import java.util.List;

import junit.framework.TestCase;
import cz.cvut.ok.kapo.paa.kybl.Configuration;
import cz.cvut.ok.kapo.paa.kybl.Node;
import cz.cvut.ok.kapo.paa.kybl.OpCodes;
import cz.cvut.ok.kapo.paa.kybl.State;
import cz.cvut.ok.kapo.paa.kybl.utils.PathPrinter;

public class BFSEngineTest extends TestCase {

	/*
	 * Test method for 'cz.cvut.ok.kapo.paa.kybl.engines.bfs.BFSEngine.getPath()'
	 */
	public void testGetPath() {
		//nastavime konfiguraci
		Configuration c = new Configuration(2,2,2,2,1);
		c.setStartState(0,0,0,0,0);
		c.setEndState(0,1,0,0,0);
		//toto by melo projit na 2 kroky, takze 3 stavy.
		//operace napln k5, prelij k5 -> k2
		BFSEngine be = new BFSEngine(c);
		be.execute();
		
		assertEquals(true, be.existPath());
		
		List<Node> l = be.getPath();
		
		assertEquals(3,l.size());
		assertEquals(OpCodes.OC_FILL, l.get(1).getOpcode());
		assertEquals(OpCodes.OC_POUR, l.get(2).getOpcode());

		//nastavime konfiguraci
		c.setStartState(0,0,0,0,0);
		c.setEndState(2,2,2,1,1);
		//toto by melo projit na 6 kroky, takze 7 stavy.
		//operace napln k1,k2,k3,k5, prelij k5 -> k4, napln k5
		be = new BFSEngine(c);
		be.execute();
		
		assertEquals(true, be.existPath());
		
		l = be.getPath();
		
		assertEquals(6,l.size());
		StringWriter sw = new StringWriter();
		PathPrinter.printPath(c, l, sw);
		System.out.println(sw.toString());
	}

	/*
	 * Test method for 'cz.cvut.ok.kapo.paa.kybl.engines.bfs.BFSEngine.getPath()'
	 */
	public void testGenerateNewStates() {
		//nastavime konfiguraci
		Configuration c = new Configuration(2,1,2,2,1);
		c.setStartState(0,0,0,0,0);
		c.setEndState(2,1,2,2,1);
		//toto by melo projit na 2 kroky, takze 3 stavy.
		//operace napln k5, prelij k5 -> k2
		BFSEngine be = new BFSEngine(c);
		
		State s = new State(1,0,0,0,0);
		//vygenerujeme stavy
		be.generateNewStates(s,be.getStateSpace().getNode(s));
		//a zkusime vyhazet ty vygenerovane
		assertEquals(true,be.pq.remove(new State(0,1,0,0,0)));
		assertEquals(true,be.pq.remove(new State(1,1,0,0,0)));
		assertEquals(true,be.pq.remove(new State(0,0,0,0,0)));
		assertEquals(true,be.pq.remove(new State(1,0,0,0,1)));
		assertEquals(true,be.pq.remove(new State(0,0,0,0,1)));
	}

}
